.Dd Sep 18, 2010
.Dt libserver 8
.Sh NAME
.Nm libserver
.Nd Multi-process TCP Server-in-a-Library
.Sh SYNOPSIS
.Nm -I/usr/local/include -L/usr/local/lib -lserver
.Sh DESCRIPTION
libserver implements a generic multi-process TCP server which is incapable
of servicing connections on its own.  The programmer writes an
initialization function and a function to service connections, then
compiles and links his or her code against the library to create a complete
multi-process TCP server.
.Sh USAGE
An example echo server is included in the libserver distribution, named
echo.c.  It is built automatically when libserver.so is built, but not
installed by the Makefile.
.Pp
The programmer must not define any global symbol beginning with the five
characters \'serv_', other than the names of the two programmer supplied
functions, because the library reserves that namespace for its own use.
.Pp
The prototypes for the two functions the programmer must supply are
declared in server.h.  You must define the functions to match the
prototypes:
.Bd -literal -offset left
void serv_init_func();
void serv_service_connection();
.Ed
.Pp
In serv_init_func(), or in functions which it calls, the programmer must
perform all initialization he or she needs to do before the server can
commence servicing connections.  serv_service_connection() is called once
for each connection.  There is one libserver library function which should
be called from inside serv_init_func() to set the server's name:
.Bd -literal -offset left
void serv_set_name( char * );
.Ed
.Pp
If not set, or set to NULL, the server's name defaults to "server".  The
server's name is used in two ways:  
.Bl -bullet
.It
When the server is running, stderr is connected to /dev/null.  Errors,
therefore, must be reported using syslog(3).  The library calls openlog()
with the server's name as argument, to ensure log entries are identified by
the server's name.
.It
The server's pidfile will be written to /var/run/, if the server is started
as root.  The filename will be the server's name with ".pid" appended to
it.  This file can be used by /usr/local/etc/rc.d/ scripts to stop the
server.  A sample script is included in the libserver distribution.
.El
.Pp
The second programmer-supplied function, serv_service_connection() is called
once for every connection accepted by the server.  The programmer may write
as many functions as he or she desires to service connections, but this is
the only one libserver calls.  The programmer must use the buffered IO
functions to communicate with the client.  serv_request_handler() should
return only when the server has finished servicing the current connection.
.Pp
Please examine the echo.c source for the example echo server.  It
demonstrates the use of libserver concisely.
.Sh CONFIGURATION
A server linked against libserver writes its pidfile into /var/run/, if
it can (ie., it is started as root), and may be stopped with a SIGTERM.  A
sample rc.d script is provided in the libserver distribution.  To use the
script, all occurrences of "server" must be replaced with the value you
pass to serv_set_name(), and the script installed in /usr/local/etc/rc.d.
Two variables must be added to /etc/rc.conf.  For example (substituting
"server" for your server's name):
.Bd -literal -offset left
server_enable="YES"
server_flags="-u www -g www -r /usr/local/server"
.Ed
.Pp
If the "enable" variable is set to "YES", the server will be started at
system start.  The following rc commands will be available:
.Bd -literal -offset left
/usr/local/etc/rc.d/server start
/usr/local/etc/rc.d/server stop
/usr/local/etc/rc.d/server restart
/usr/local/etc/rc.d/server status
.Ed
.Pp
If you do not want the server started on system start, then set
.Bd -literal -offset left
server_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/server forcestart
/usr/local/etc/rc.d/server forcestop
/usr/local/etc/rc.d/server forcerestart
/usr/local/etc/rc.d/server forcestatus
.Ed
.Pp
.Ss COMMAND-LINE OPTIONS
The following command-line options are recognized by libserver servers.
All of these are optional.
.Bl -tag -width "-r"
.It -r
The -r option specifies the server root directory, which libserver will make
its current working directory.
.It -p
The -p option specifies the port to listen on.  This defaults to 1966 if
not specified.  Note that to bind to a port lower than 1024, the server
must be started as root.
.It -i
By default, libserver accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting libserver to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.
.It -m
.It -n
libserver uses a multi-process model, with one master process forking or
killing slave processes in response to client activity.  Only the slave
processes service client requests.  The -n options specifies the number of
idle slaves the master will attempt to keep ready at all times.  The -m
option specifies the maximum number of slaves which may be running at any
time.  The value specified must be equal to or greater than the value
specified for -n.  The default value for -n is 3 and the default value for
-m is 25.
.It -u
.It -g
The -u and the -g options may be used to specify the user and group for
the server to change to after it has bound to the listening socket.  If not
specified, both values default to "nobody".  Note that in order for a
libserver server to change user the server must be started as root.  If not
started as root, two error messages will be syslog()ed at start-up,
complaining about the inability of the server to change user and group.
You can suppress them by providing values to -u and -g to override the
default "nobody" with the actual user and group under which the server
runs.
.It -x
The -x option, if present, prevents libserver from becoming a daemon.  It
will then run in the foreground of the terminal where it was started, and
may be stopped with signals (ie., Control-C).  Stderr will be connected to
the terminal, so diagnostic output may be sent there when this option is
present on the command-line.  The server also will not write its pidfile to
/var/run/ when the -x option is used.
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
